@@ -1,5 +1,6 @@
 from __future__ import division
 
+import logging
 from numpy import asarray, concatenate
 from scipy.optimize import fmin_l_bfgs_b
 
@@ -19,6 +20,7 @@ class ProxyFunction(object):
         self._signal = -1 if negative else +1
         self.progress = progress
         self._solutions = []
+        self._logger = logging.getLogger(__name__)
 
     @property
     def solutions(self):
@@ -32,7 +34,12 @@ class ProxyFunction(object):
 
     def gradient(self):
         g = self._function.gradient()
-        return {name: self._signal * g[name] for name in self.names()}
+        grad = {name: self._signal * g[name] for name in self.names()}
+
+        if self._logger.getEffectiveLevel() <= logging.DEBUG:
+            self._logger.debug("Gradient: %s", str(grad))
+
+        return grad
 
     def unflatten(self, x):
         variables = self._function.variables().select(fixed=False)
@@ -49,11 +56,22 @@ class ProxyFunction(object):
         return do_flatten([d[name] for name in names])
 
     def __call__(self, x):
+
         x = asarray(x).ravel()
         self._solutions.append(x.copy())
         self._function.variables().set(self.unflatten(x))
+
+        if self._logger.getEffectiveLevel() <= logging.DEBUG:
+            var = self._function.variables().select(fixed=False)
+            for name in self.names():
+                self._logger.debug("Setting %s to %s", name, var[name])
+
         v = self.value()
         g = self.flatten(self.gradient())
+
+        if self._logger.getEffectiveLevel() <= logging.DEBUG:
+            self._logger.debug("Function evaluation is %.10f", v)
+
         return v, g
 
     def set_solution(self, x):
@@ -66,15 +84,27 @@ class ProxyFunction(object):
 
 def _try_minimize(proxy_function, n):
     disp = 1 if proxy_function.progress else 0
+    logger = logging.getLogger()
 
     if n == 0:
         raise OptimixError("Too many bad solutions")
 
     try:
         x0 = proxy_function.get_solution()
-        return fmin_l_bfgs_b(proxy_function, x0, disp=disp)
 
-    except BadSolutionError:
+        bounds = []
+
+        var = proxy_function._function.variables().select(fixed=False)
+        for name in proxy_function.names():
+            if len(var[name].shape) == 0:
+                bounds.append(var[name].bounds)
+            else:
+                bounds += var[name].bounds
+
+        res = fmin_l_bfgs_b(proxy_function, x0, bounds=bounds,
+                             disp=disp)
+
+    except BadSolutionError as err:
 
         xs = proxy_function.solutions
         if len(xs) < 2:
@@ -82,9 +112,10 @@ def _try_minimize(proxy_function, n):
 
         proxy_function.set_solution(xs[-2] / 5 + xs[-1] / 5)
 
-        if disp == 1:
-            print("Optimix: Restarting L-BFGS-B due to bad solution.")
-        return _try_minimize(proxy_function, n - 1)
+        logger.info("Optimix: Restarting L-BFGS-B due to bad solution.")
+        res = _try_minimize(proxy_function, n - 1)
+
+    return res
 
 
 def _minimize(proxy_function):
