@@ -1,9 +1,8 @@
-import sys
-
+"""Wrapper for Brent search."""
 from numpy import asarray
+from tqdm import tqdm
 
 from brent_search import minimize as brent_minimize
-from tqdm import tqdm
 
 
 class ProxyFunction(object):
@@ -13,30 +12,43 @@ class ProxyFunction(object):
         self._progress = tqdm(desc='Optimix', disable=not progress)
         self._iteration = 0
 
+    def names(self):
+        return sorted(self._function.variables().select(fixed=False).names())
+
     def value(self):
         return self._signal * self._function.value()
 
+    def unflatten(self, x):
+        variables = self._function.variables().select(fixed=False)
+        d = dict()
+        offset = 0
+        for name in self.names():
+            size = variables.get(name).size
+            d[name] = x[offset:offset + size]
+            offset += size
+        return d
+
     def __call__(self, x):
         self._progress.update(1)
 
         self._iteration += 1
 
         x = asarray(x).ravel()
-        self._function.variables().from_flat(x)
+        self._function.variables().set(self.unflatten(x))
 
         return self.value()
 
     def set_solution(self, x):
-        self._function.variables().from_flat(asarray(x).ravel())
+        self._function.variables().set(self.unflatten(x))
         self._progress.close()
 
 
 def _minimize(proxy_function):
-    x = brent_minimize(proxy_function)
-    proxy_function.set_solution(x)
+    x = asarray(brent_minimize(proxy_function))
+    proxy_function.set_solution(x[0:1])
 
 
-def minimize(function, progress=False):
+def minimize(function, progress=True):
     return _minimize(ProxyFunction(function, progress, False))
 
 
